/*
 * Copyright 2013 Andy Boothe
 * 
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 * 
 *        http://www.apache.org/licenses/LICENSE-2.0
 * 
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.sigpwned.jsonification;

import java.io.IOException;

public interface JsonProcessor<T> {
	/**
	 * Represents either the name of a JsonValue within a JsonObject, or the
	 * index of a JsonValue within a JsonList.
	 */
	public static interface Coordinate {
		/**
		 * If the value being handled is within a JsonList, then this will
		 * return the index of the value within that JsonList. If this value
		 * is not null, then getName() will be null.
		 */
		public Integer getIndex();
		
		/**
		 * If the value being handled is within a JsonObject, then this will
		 * return the name of the value within that JsonObject. If this value
		 * is not null, then getIndex() will be null.
		 */
		public String getName();
	}
	
	public static class Index implements Coordinate {
		private int index;
		
		public Index(int index) {
			this.index = index;
		}
		
		public String getName() {
			return null;
		}
		
		public Integer getIndex() {
			return index;
		}
	}
	
	public static class Name implements Coordinate {
		private String name;
		
		public Name(String name) {
			this.name = name;
		}
		
		public String getName() {
			return name;
		}
		
		public Integer getIndex() {
			return null;
		}
	}
	
	
	public void beginJson() throws IOException;
	
	public T endJson() throws IOException;
	
	public void beginObject(Coordinate coordinate) throws IOException;
	
	public void endObject() throws IOException;
	
	public void beginList(Coordinate coordinate) throws IOException;
	
	public void endList() throws IOException;
	
	public void value(Coordinate coordinate, boolean value) throws IOException;

	public void value(Coordinate coordinate, long value) throws IOException;

	public void value(Coordinate coordinate, double value) throws IOException;

	public void value(Coordinate coordinate, String value) throws IOException;

	public void nullValue(Coordinate coordinate) throws IOException;
}
