/*
 * Copyright 2013 Andy Boothe
 * 
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 * 
 *        http://www.apache.org/licenses/LICENSE-2.0
 * 
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.sigpwned.jsonification;

import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.List;

import com.sigpwned.jsonification.manipulation.JsonDocumentBuilder;
import com.sigpwned.jsonification.manipulation.JsonFragmentBuilder;
import com.sigpwned.jsonification.serialization.JsonSerializationException;
import com.sigpwned.jsonification.serialization.serialize.JsonSerializeApproach;
import com.sigpwned.jsonification.serialization.serialize.JsonTypeSerializer;

public interface JsonSerializer {
	public static interface ExceptionPolicy {
		public static final ExceptionPolicy IGNORE=new ExceptionPolicy() {
			public JsonSerializationException format(IllegalArgumentException e) {
				return null;
			}
				
			public JsonSerializationException format(IllegalAccessException e) {
				return null;
			}
				
			public JsonSerializationException format(InvocationTargetException e) {
				return null;
			}
				
			public JsonSerializationException emptyBean(Object bean) {
				return null;
			}
		};

		public static final ExceptionPolicy THROW=new ExceptionPolicy() {
			public JsonSerializationException format(IllegalArgumentException e) {
				return new JsonSerializationException(e);
			}
				
			public JsonSerializationException format(IllegalAccessException e) {
				return new JsonSerializationException(e);
			}
				
			public JsonSerializationException format(InvocationTargetException e) {
				return new JsonSerializationException(e);
			}
				
			public JsonSerializationException emptyBean(Object bean) {
				return new JsonSerializationException("Could not serialize any fields for value: "+bean);
			}
		};
		
		public JsonSerializationException format(IllegalArgumentException e);
		public JsonSerializationException format(IllegalAccessException e);
		public JsonSerializationException format(InvocationTargetException e);
		public JsonSerializationException emptyBean(Object bean);
	}

	public ExceptionPolicy getExceptionPolicy();
	
	public List<JsonSerializeApproach> getSerializeApproaches();
	
	public void addSerializeApproach(JsonSerializeApproach serializeApproach);
	
	public void addTypeSerializer(JsonTypeSerializer<?> typeSerializer);
	
	public boolean hasTypeSerializer(Class<?> type);
	
	public JsonTypeSerializer<?> getTypeSerializer(Class<?> type);

	public <T> T serialize(JsonDocumentBuilder<T> builder, Object value) throws IOException;
	
	public void serializeFragment(JsonFragmentBuilder builder, Object value) throws IOException;
	
}
