/*
 * Copyright 2013 Andy Boothe
 * 
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 * 
 *        http://www.apache.org/licenses/LICENSE-2.0
 * 
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.sigpwned.jsonification.accessor;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import com.sigpwned.jsonification.accessor.processing.AllIndexes;
import com.sigpwned.jsonification.accessor.processing.RangeIndexes;
import com.sigpwned.jsonification.accessor.processing.SingleIndexes;
import com.sigpwned.jsonification.util.Parser;
import com.sigpwned.jsonification.x.JsonParseException;

public class AccessorParser extends Parser<Void> {
	private StringBuilder buf;
	private AccessorProcessor processor;
	
	public AccessorParser(AccessorProcessor processor) {
		this.buf = new StringBuilder();
		this.processor = processor;
	}

	protected StringBuilder getBuf() {
		return buf;
	}

	protected AccessorProcessor getProcessor() {
		return processor;
	}

	@Override
	protected Void parse() throws IOException {
		getProcessor().beginAccessor();
		try {
			whitespace();
			
			while(peek() != -1) {
				if(attempt('.')) {
					List<String> names=new ArrayList<String>();
					names.add(id());
					while(attempt(','))
						names.add(id());
					getProcessor().objectAccess(names);
				} else
				if(attempt('[')) {
					AccessorProcessor.Indexes indexes;
					
					if(attempt('*'))
						indexes = new AllIndexes();
					else {
						int index1=digits();
						
						Integer index2;
						if(attempt('.')) {
							consume('.');
							index2 = digits();
						}
						else
							index2 = null;

						if(index2 != null)
							indexes = new RangeIndexes(index1, index2);
						else
							indexes = new SingleIndexes(index1);
					}
					
					consume(']');
					
					getProcessor().arrayAccess(indexes);
				}
				else
					throw new JsonParseException("Unrecognized character: "+((char) peek()));
			}
		}
		finally {
			getProcessor().endAccessor();
		}
		return null;
	}
	
	protected String id() throws IOException {
		if(peek() == -1)
			throw new JsonParseException("Unexpected EOF");
		
		if(!Character.isJavaIdentifierStart(peek()))
			throw new JsonParseException("Unexpected character: "+((char) peek()));
		
		getBuf().setLength(0);
		getBuf().append((char) read());
		while(Character.isJavaIdentifierPart(peek()))
			getBuf().append((char) read());
		
		return getBuf().toString();
	}
	
	protected int digits() throws IOException {
		getBuf().setLength(0);
		while(Character.isDigit(peek()))
			getBuf().append((char) read());
		return Integer.parseInt(getBuf().toString());
	}
}
