/*
 * Copyright 2013 Andy Boothe
 * 
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 * 
 *        http://www.apache.org/licenses/LICENSE-2.0
 * 
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.sigpwned.jsonification.accessor.reference;

import java.util.ArrayList;
import java.util.List;

import com.sigpwned.jsonification.Accessor;
import com.sigpwned.jsonification.accessor.JsonReference;
import com.sigpwned.jsonification.accessor.x.InvalidObjectJsonAccessorEvalException;
import com.sigpwned.jsonification.model.JsonObject;
import com.sigpwned.jsonification.model.JsonValue;

public class ObjectSlotJsonReference extends AbstractJsonReference {
	private JsonReference object;
	private List<String> names;
	
	public ObjectSlotJsonReference(JsonReference object, List<String> names) {
		this.object = object;
		this.names   = names;
	}
	
	public List<JsonValue> get(JsonValue root) {
		List<JsonValue> result=new ArrayList<JsonValue>();
		
		List<JsonValue> values=getObject().get(root);
		for(JsonValue value : values) {
			JsonObject o=value.asObject();
			for(String name : getNames())
				if(!o.containsKey(name))
					throw new InvalidObjectJsonAccessorEvalException(o, name);
				else
					result.add(o.get(name));
		}
		
		return result;
	}
	
	public void update(JsonValue root, Accessor.Function transformation) {
		List<JsonValue> values=getObject().get(root);
		for(JsonValue value : values) {
			JsonObject o=value.asObject();
			for(String name : getNames())
				if(!o.containsKey(name))
					throw new InvalidObjectJsonAccessorEvalException(o, name);
				else
					o.put(name, value);
		}		
	}
	
	public JsonReference getObject() {
		return object;
	}

	public List<String> getNames() {
		return names;
	}

	public void append(StringBuilder buf) {
		getObject().append(buf);
		buf.append(".");
		buf.append(getNames().get(0));
		for(int i=1;i<getNames().size();i++) {
			buf.append(",");
			buf.append(getNames().get(i));
		}
	}

	public Multiplicity getMultiplicity() {
		return getNames().size()==1 ? Multiplicity.ONE : Multiplicity.MANY;
	}
}