/*
 * Copyright 2013 Andy Boothe
 * 
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 * 
 *        http://www.apache.org/licenses/LICENSE-2.0
 * 
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.sigpwned.jsonification.manipulation;

import java.io.IOException;

import com.sigpwned.jsonification.JsonProcessor;
import com.sigpwned.jsonification.JsonProcessor.Coordinate;



public interface JsonDocumentBuilder<T> extends JsonFragmentBuilder {
    public JsonDocumentBuilder<T> name(String name) throws IOException;

    public JsonDocumentBuilder<T> beginObject() throws IOException;

    public JsonDocumentBuilder<T> beginObject(String name) throws IOException;

    public JsonDocumentBuilder<T> beginObject(Coordinate coordinate) throws IOException;

    public JsonDocumentBuilder<T> endObject() throws IOException;

    public JsonDocumentBuilder<T> beginList() throws IOException;

    public JsonDocumentBuilder<T> beginList(String name) throws IOException;

    public JsonDocumentBuilder<T> beginList(JsonProcessor.Coordinate coordinate) throws IOException;

    public JsonDocumentBuilder<T> endList() throws IOException;

    public JsonDocumentBuilder<T> value(long value) throws IOException;

    public JsonDocumentBuilder<T> value(String name, long value) throws IOException;

    public JsonDocumentBuilder<T> value(JsonProcessor.Coordinate coordinate, long value) throws IOException;

    public JsonDocumentBuilder<T> value(double value) throws IOException;

    public JsonDocumentBuilder<T> value(String name, double value) throws IOException;

    public JsonDocumentBuilder<T> value(JsonProcessor.Coordinate coordinate, double value) throws IOException;

    public JsonDocumentBuilder<T> value(String value) throws IOException;

    public JsonDocumentBuilder<T> value(String name, String value) throws IOException;

    public JsonDocumentBuilder<T> value(JsonProcessor.Coordinate coordinate, String value) throws IOException;

    public JsonDocumentBuilder<T> value(boolean value) throws IOException;

    public JsonDocumentBuilder<T> value(String name, boolean value) throws IOException;

    public JsonDocumentBuilder<T> value(JsonProcessor.Coordinate coordinate, boolean value) throws IOException;

    public JsonDocumentBuilder<T> nullValue() throws IOException;

    public JsonDocumentBuilder<T> nullValue(String name) throws IOException;

    public JsonDocumentBuilder<T> nullValue(Coordinate coordinate) throws IOException;
    
	public T end() throws IOException;
}