/*
 * Copyright 2013 Andy Boothe
 * 
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 * 
 *        http://www.apache.org/licenses/LICENSE-2.0
 * 
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.sigpwned.jsonification.manipulation;

import java.io.IOException;

import com.sigpwned.jsonification.JsonProcessor;
import com.sigpwned.jsonification.JsonProcessor.Coordinate;

public interface JsonFragmentBuilder {
	public JsonFragmentBuilder name(String name) throws IOException;

	public JsonFragmentBuilder beginObject() throws IOException;

	public JsonFragmentBuilder beginObject(String name) throws IOException;

	public JsonFragmentBuilder beginObject(Coordinate coordinate) throws IOException;

	public JsonFragmentBuilder endObject() throws IOException;

	public JsonFragmentBuilder beginList() throws IOException;

	public JsonFragmentBuilder beginList(String name) throws IOException;

	public JsonFragmentBuilder beginList(JsonProcessor.Coordinate coordinate) throws IOException;

	public JsonFragmentBuilder endList() throws IOException;

	public JsonFragmentBuilder value(long value) throws IOException;

	public JsonFragmentBuilder value(String name, long value) throws IOException;

	public JsonFragmentBuilder value(JsonProcessor.Coordinate coordinate, long value) throws IOException;

	public JsonFragmentBuilder value(double value) throws IOException;

	public JsonFragmentBuilder value(String name, double value) throws IOException;

	public JsonFragmentBuilder value(JsonProcessor.Coordinate coordinate, double value) throws IOException;

	public JsonFragmentBuilder value(String value) throws IOException;

	public JsonFragmentBuilder value(String name, String value) throws IOException;

	public JsonFragmentBuilder value(JsonProcessor.Coordinate coordinate, String value) throws IOException;

	public JsonFragmentBuilder value(boolean value) throws IOException;

	public JsonFragmentBuilder value(String name, boolean value) throws IOException;

	public JsonFragmentBuilder value(JsonProcessor.Coordinate coordinate, boolean value) throws IOException;

	public JsonFragmentBuilder nullValue() throws IOException;

	public JsonFragmentBuilder nullValue(String name) throws IOException;

	public JsonFragmentBuilder nullValue(Coordinate coordinate) throws IOException;
}