/*
 * Copyright 2013 Andy Boothe
 * 
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 * 
 *        http://www.apache.org/licenses/LICENSE-2.0
 * 
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.sigpwned.jsonification.manipulation.build.processing;

import com.sigpwned.jsonification.JsonProcessor;
import com.sigpwned.jsonification.manipulation.build.processing.coordinates.ListCoordinateFactory;
import com.sigpwned.jsonification.manipulation.build.processing.coordinates.ObjectCoordinateFactory;

public abstract class CoordinateFactory {
    private boolean poppable;
    
    public CoordinateFactory(boolean poppable) {
        this.poppable = poppable;
    }
    
    public boolean isPoppable() {
        return poppable;
    }
    
	public abstract void primeCoordinate(Object value);
	
	public abstract JsonProcessor.Coordinate currentCoordinate();
	
	public abstract JsonProcessor.Coordinate nextCoordinate(Object value);
	
    public abstract ScopeType getScopeType();
    
    public String getScopeTypeName() {
        return getScopeType().getScopeTypeName();
    }
    
	public ObjectCoordinateFactory asObject() {
		return (ObjectCoordinateFactory) this;
	}
	
	public ListCoordinateFactory asList() {
		return (ListCoordinateFactory) this;
	}
}