/*
 * Copyright 2013 Andy Boothe
 * 
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 * 
 *        http://www.apache.org/licenses/LICENSE-2.0
 * 
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.sigpwned.jsonification.manipulation.build.processing;

import java.io.IOException;

import com.sigpwned.jsonification.manipulation.build.ProcessingJsonDocumentBuilder;
import com.sigpwned.jsonification.manipulation.build.x.IllegalEventJsonBuildingException;

public enum State {
	UNINITIALIZED {
		@Override
		public <T> State step(ProcessingJsonDocumentBuilder<T> builder) throws IOException {
			builder.getProcessor().beginJson();
			return State.READY;
		}

		@Override
		public <T> T complete(ProcessingJsonDocumentBuilder<T> builder) throws IOException {
			builder.getProcessor().beginJson();
			return builder.getProcessor().endJson();
		}
	},
	READY {
		@Override
		public <T> State step(ProcessingJsonDocumentBuilder<T> builder) {
			return State.READY;
		}

		@Override
		public <T> T complete(ProcessingJsonDocumentBuilder<T> builder) throws IOException {
			return builder.getProcessor().endJson();
		}
	},
	DONE {
		@Override
		public <T> State step(ProcessingJsonDocumentBuilder<T> builder) {
			throw new IllegalEventJsonBuildingException("Cannot build JSON (already built JSON)");
		}

		@Override
		public <T> T complete(ProcessingJsonDocumentBuilder<T> builder) {
			throw new IllegalEventJsonBuildingException("Cannot build JSON (already built JSON)");
		}
	};
	
	public abstract <T> State step(ProcessingJsonDocumentBuilder<T> builder) throws IOException;
	
	public abstract <T> T complete(ProcessingJsonDocumentBuilder<T> builder) throws IOException;
}