/*
 * Copyright 2013 Andy Boothe
 * 
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 * 
 *        http://www.apache.org/licenses/LICENSE-2.0
 * 
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.sigpwned.jsonification.manipulation.build.processing.coordinates;

import com.sigpwned.jsonification.JsonProcessor;
import com.sigpwned.jsonification.JsonProcessor.Coordinate;
import com.sigpwned.jsonification.manipulation.build.processing.CoordinateFactory;
import com.sigpwned.jsonification.manipulation.build.processing.ScopeType;
import com.sigpwned.jsonification.manipulation.build.x.IllegalCoordinateJsonBuildingException;

public class ListCoordinateFactory extends CoordinateFactory {
	private JsonProcessor.Index currentCoordinate;
	private int index;
	
    public ListCoordinateFactory(boolean poppable) {
        this(poppable, 0);
    }
    
    public ListCoordinateFactory(boolean poppable, int index) {
	    super(poppable);
		this.index = index;
	}
	
	@Override
	public void primeCoordinate(Object value) {
		throw new IllegalCoordinateJsonBuildingException("Cannot prime list index");
	}
	
	@Override
	public Coordinate currentCoordinate() {
		return currentCoordinate;
	}

	@Override
	public JsonProcessor.Coordinate nextCoordinate(Object value) {
		JsonProcessor.Index result;
		
		if(value != null)
			throw new IllegalCoordinateJsonBuildingException("Cannot provide list index");
		
		currentCoordinate = result = new JsonProcessor.Index(nextIndex());
		
		return result;
	}

	@Override
	public ScopeType getScopeType() {
		return ScopeType.LIST;
	}
	
	protected int nextIndex() {
		return index++;
	}
}