/*
 * Copyright 2013 Andy Boothe
 * 
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 * 
 *        http://www.apache.org/licenses/LICENSE-2.0
 * 
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.sigpwned.jsonification.manipulation.build.processing.coordinates;

import com.sigpwned.jsonification.JsonProcessor;
import com.sigpwned.jsonification.manipulation.build.processing.CoordinateFactory;
import com.sigpwned.jsonification.manipulation.build.processing.ScopeType;
import com.sigpwned.jsonification.manipulation.build.x.IllegalCoordinateJsonBuildingException;

public class ObjectCoordinateFactory extends CoordinateFactory {
	private JsonProcessor.Name currentCoordinate;
	private String nextName;
	
	public ObjectCoordinateFactory(boolean poppable) {
	    super(poppable);
		this.nextName = null;
	}

	@Override
	public void primeCoordinate(Object value) {
		this.nextName = name(value);
	}
	
	@Override
	public JsonProcessor.Coordinate currentCoordinate() {
		return currentCoordinate;
	}

	@Override
	public JsonProcessor.Coordinate nextCoordinate(Object nextName0) {
		JsonProcessor.Name result;
		
		String nextName=name(nextName0);
		
		if(nextName!=null && this.nextName!=null)
			throw new IllegalCoordinateJsonBuildingException("Specified both primed and next coordinates: `"+this.nextName+"', `"+nextName+"'");
		else
		if(nextName != null)
			result = new JsonProcessor.Name(nextName);
		else
		if(this.nextName != null)
			result = new JsonProcessor.Name(this.nextName);
		else
			result = null;
		
		this.currentCoordinate = result;
		
		this.nextName = null;
		
		return result;
	}
	
	@Override
	public ScopeType getScopeType() {
		return ScopeType.OBJECT;
	}
	
	protected String name(Object value) {
		String result;
		try {
			result = (String) value;
		}
		catch(ClassCastException e) {
			throw new IllegalCoordinateJsonBuildingException("Not a valid name: "+value+" (not a String)");
		}
		return result;
	}
}