/*
 * Copyright 2013 Andy Boothe
 * 
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 * 
 *        http://www.apache.org/licenses/LICENSE-2.0
 * 
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.sigpwned.jsonification.model.impl;

import com.sigpwned.jsonification.model.JsonFloat;
import com.sigpwned.jsonification.model.JsonNumber;
import com.sigpwned.jsonification.model.x.PrecisionLossJsonException;

public class JsonFloatImpl extends JsonValueImpl implements JsonFloat {
	public static final JsonFloat ZERO=new JsonFloatImpl(0.0);
	public static final JsonFloat ONE=new JsonFloatImpl(1.0);
	
	private double value;
	
	public JsonFloatImpl(double value) {
		this.value = value;
	}
	
	public JsonFloat asFloat() {
		return this;
	}
	
	public JsonNumber asNumber() {
		Long longVal;
		if(doubleVal()<Long.MIN_VALUE || doubleVal()>Long.MAX_VALUE)
			longVal = null;
		else
			longVal = (long) doubleVal();
		return new JsonNumberImpl(JsonNumber.OriginalType.FLOAT, longVal, doubleVal());
	}

	public float floatVal() {
		if(doubleVal()<-Float.MAX_VALUE || doubleVal()>Float.MAX_VALUE)
			throw new PrecisionLossJsonException();
		return (float) doubleVal();
	}

	public double doubleVal() {
		return value;
	}
}
