/*
 * Copyright 2013 Andy Boothe
 * 
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 * 
 *        http://www.apache.org/licenses/LICENSE-2.0
 * 
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.sigpwned.jsonification.model.impl;

import com.sigpwned.jsonification.model.JsonInt;
import com.sigpwned.jsonification.model.JsonNumber;
import com.sigpwned.jsonification.model.x.PrecisionLossJsonException;

public class JsonIntImpl extends JsonValueImpl implements JsonInt {
	public static final JsonInt ZERO=new JsonIntImpl(0L);
	public static final JsonInt ONE=new JsonIntImpl(1L);
	
	private long value;
	
	public JsonIntImpl(long value) {
		this.value = value;
	}
	
	public JsonInt asInt() {
		return this;
	}
	
	public JsonNumber asNumber() {
		return new JsonNumberImpl(JsonNumber.OriginalType.INT, longVal(), (double) longVal());
	}

	public byte byteVal() {
		if(longVal()<Byte.MIN_VALUE || longVal()>Byte.MAX_VALUE)
			throw new PrecisionLossJsonException();
		return (byte) longVal();
	}

	public short shortVal() {
		if(longVal()<Short.MIN_VALUE || longVal()>Short.MAX_VALUE)
			throw new PrecisionLossJsonException();
		return (short) longVal();
	}

	public int intVal() {
		if(longVal()<Integer.MIN_VALUE || longVal()>Integer.MAX_VALUE)
			throw new PrecisionLossJsonException();
		return (int) longVal();
	}

	public long longVal() {
		return value;
	}
}
