/*
 * Copyright 2013 Andy Boothe
 * 
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 * 
 *        http://www.apache.org/licenses/LICENSE-2.0
 * 
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.sigpwned.jsonification.model.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.ListIterator;

import com.sigpwned.jsonification.model.JsonList;
import com.sigpwned.jsonification.model.JsonValue;
import com.sigpwned.jsonification.model.x.IndexOutOfBoundsJsonException;

public class JsonListImpl extends JsonValueImpl implements JsonList {
	private List<JsonValue> values;
	
	public JsonListImpl() {
		this.values = new ArrayList<JsonValue>();
	}
	
	public JsonList asList() {
		return this;
	}
	
	private List<JsonValue> getValues() {
		return values;
	}
	
	private int check(int index, boolean endok) {
		if(index<0 || index>size() || (!endok && index==size()))
			throw new IndexOutOfBoundsJsonException(index);
		return index;
	}

	public int size() {
		return getValues().size();
	}

	public boolean isEmpty() {
		return getValues().isEmpty();
	}

	public void add(JsonValue value) {
		if(value == null)
			throw new NullPointerException();
		getValues().add(value);
	}

	public void addAll(JsonList values) {
		if(values == null)
			throw new NullPointerException();
		for(JsonValue value : values)
			getValues().add(value);
	}

	public void clear() {
		getValues().clear();
	}

	public JsonValue remove(int index) {
		return getValues().remove(check(index, false));
	}

	public JsonValue get(int index) {
		return getValues().get(check(index, false));
	}

	public JsonValue set(int index, JsonValue value) {
		return getValues().set(check(index, true), value);
	}

	public ListIterator<JsonValue> listIterator() {
		return getValues().listIterator();
	}

	public ListIterator<JsonValue> listIterator(int index) {
		return getValues().listIterator(check(index, true));
	}

	public Iterator<JsonValue> iterator() {
		return getValues().iterator();
	}
}
