/*
 * Copyright 2013 Andy Boothe
 * 
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 * 
 *        http://www.apache.org/licenses/LICENSE-2.0
 * 
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.sigpwned.jsonification.model.impl;

import com.sigpwned.jsonification.model.JsonFloat;
import com.sigpwned.jsonification.model.JsonInt;
import com.sigpwned.jsonification.model.JsonNumber;
import com.sigpwned.jsonification.model.x.PrecisionLossJsonException;
import com.sigpwned.jsonification.x.InternalErrorJsonException;

public class JsonNumberImpl extends JsonValueImpl implements JsonNumber {
	private static void check(OriginalType originalType, Long longVal, Double doubleVal) {
		if(originalType == null)
			throw new NullPointerException();
		else
		if(originalType == OriginalType.FLOAT) {
			if(doubleVal == null)
				throw new NullPointerException();
		} else
		if(originalType == OriginalType.INT) {
			if(doubleVal == null)
				throw new NullPointerException();
			if(longVal == null)
				throw new NullPointerException();
		}
		else
			throw new InternalErrorJsonException("Unhandled OriginalType: "+originalType);
	}
	
	private OriginalType originalType;
	private Long longVal;
	private Double doubleVal;
	
	/* default */ JsonNumberImpl(OriginalType originalType, Long longVal, Double doubleVal) {
		check(originalType, longVal, doubleVal);
		this.originalType = originalType;
		this.longVal = longVal;
		this.doubleVal = doubleVal;
	}
	
	public OriginalType getOriginalType() {
		return originalType;
	}
	
	public JsonFloat asFloat() {
		return new JsonFloatImpl(doubleVal());
	}
	
	public JsonInt asInt() {
		return new JsonIntImpl(longVal());
	}
	
	public JsonNumber asNumber() {
		return this;
	}

	public byte byteVal() {
		if(longVal()<Byte.MIN_VALUE || longVal()>Byte.MAX_VALUE)
			throw new PrecisionLossJsonException();
		return (byte) longVal();
	}

	public short shortVal() {
		if(longVal()<Short.MIN_VALUE || longVal()>Short.MAX_VALUE)
			throw new PrecisionLossJsonException();
		return (short) longVal();
	}

	public int intVal() {
		if(longVal()<Integer.MIN_VALUE || longVal()>Integer.MAX_VALUE)
			throw new PrecisionLossJsonException();
		return (int) longVal();
	}

	public long longVal() {
		if(longVal == null)
			throw new PrecisionLossJsonException();
		return longVal;
	}

	public float floatVal() {
		if(doubleVal()<-Float.MAX_VALUE || doubleVal()>Float.MAX_VALUE)
			throw new PrecisionLossJsonException();
		return (float) doubleVal();
	}

	public double doubleVal() {
		if(doubleVal == null) {
			// This should never happen
			throw new PrecisionLossJsonException();
		}
		return doubleVal;
	}
}
