/*
 * Copyright 2013 Andy Boothe
 * 
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 * 
 *        http://www.apache.org/licenses/LICENSE-2.0
 * 
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.sigpwned.jsonification.model.impl;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import com.sigpwned.jsonification.model.JsonObject;
import com.sigpwned.jsonification.model.JsonValue;
import com.sigpwned.jsonification.model.x.UndefinedKeyJsonException;

public class JsonObjectImpl extends JsonValueImpl implements JsonObject {
	private Map<String,JsonValue> values;
	
	private Map<String,JsonValue> getValues() {
		return values;
	}
	
	public JsonObjectImpl() {
		this.values = new HashMap<String,JsonValue>();
	}
	
	public JsonObject asObject() {
		return this;
	}

	public Set<String> keySet() {
		return getValues().keySet();
	}

	public Collection<JsonValue> values() {
		return getValues().values();
	}

	public boolean containsKey(String name) {
		if(name == null)
			throw new NullPointerException();
		return getValues().containsKey(name);
	}

	public int size() {
		return getValues().size();
	}

	public boolean isEmpty() {
		return getValues().isEmpty();
	}

	public JsonValue get(String name) {
		if(name == null)
			throw new NullPointerException();
		if(!containsKey(name))
			throw new UndefinedKeyJsonException(name);
		return getValues().get(name);
	}

	public JsonValue get(String name, JsonValue defaultValue) {
		if(name == null)
			throw new NullPointerException();
		if(defaultValue == null)
			throw new NullPointerException();
		JsonValue result;
		try {
			result = get(name);
		}
		catch(UndefinedKeyJsonException _) {
			result = defaultValue;
		}
		return result;
	}

	public JsonValue put(String name, JsonValue value) {
		if(name == null)
			throw new NullPointerException();
		if(value == null)
			throw new NullPointerException();
		return getValues().put(name, value);
	}

	public JsonValue remove(String name) {
		if(name == null)
			throw new NullPointerException();
		if(!containsKey(name))
			throw new UndefinedKeyJsonException(name);
		return getValues().remove(name);
	}

	public void putAll(JsonObject other) {
		for(String key : other.keySet())
			put(key, other.get(key));
	}
}
