/*
 * Copyright 2013 Andy Boothe
 * 
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 * 
 *        http://www.apache.org/licenses/LICENSE-2.0
 * 
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.sigpwned.jsonification.model.impl;

import java.util.List;

import com.sigpwned.jsonification.Accessor;
import com.sigpwned.jsonification.model.JsonBoolean;
import com.sigpwned.jsonification.model.JsonFloat;
import com.sigpwned.jsonification.model.JsonInt;
import com.sigpwned.jsonification.model.JsonList;
import com.sigpwned.jsonification.model.JsonNumber;
import com.sigpwned.jsonification.model.JsonObject;
import com.sigpwned.jsonification.model.JsonString;
import com.sigpwned.jsonification.model.JsonValue;
import com.sigpwned.jsonification.model.x.ClassCastJsonException;

public class JsonValueImpl implements JsonValue {
	public JsonInt asInt() {
		throw new ClassCastJsonException(JsonInt.class, this);
	}

	public JsonFloat asFloat() {
		throw new ClassCastJsonException(JsonFloat.class, this);
	}

	public JsonNumber asNumber() {
		throw new ClassCastJsonException(JsonNumber.class, this);
	}

	public JsonString asString() {
		throw new ClassCastJsonException(JsonString.class, this);
	}

	public JsonBoolean asBoolean() {
		throw new ClassCastJsonException(JsonBoolean.class, this);
	}

	public JsonObject asObject() {
		throw new ClassCastJsonException(JsonObject.class, this);
	}

	public JsonList asList() {
		throw new ClassCastJsonException(JsonList.class, this);
	}
    
    public List<JsonValue> readMany(Accessor accessor) {
        return accessor.readMany(this);
    }

    public List<JsonValue> readMany(String accessor) {
        return readMany(Accessor.parse(accessor));
    }
    
    public JsonValue readOne(Accessor accessor) {
        return accessor.readOne(this);
    }

    public JsonValue readOne(String accessor) {
        return readOne(Accessor.parse(accessor));
    }
    
    public void writeMany(Accessor accessor, Accessor.Function transformation) {
        accessor.writeMany(this, transformation);
    }

    public void writeMany(String accessor, Accessor.Function transformation) {
        writeMany(Accessor.parse(accessor), transformation);
    }

    public void writeMany(Accessor accessor, Accessor.Function transformation, boolean failureAcceptable) {
        accessor.writeMany(this, transformation, failureAcceptable);
    }

    public void writeMany(String accessor, Accessor.Function transformation, boolean failureAcceptable) {
        writeMany(Accessor.parse(accessor), transformation, failureAcceptable);
    }

    public void writeOne(Accessor accessor, JsonValue value) {
        accessor.writeOne(this, value);
    }

    public void writeOne(String accessor, JsonValue value) {
        writeOne(Accessor.parse(accessor), value);
    }

    public void writeOne(Accessor accessor, Accessor.Function transformation) {
        accessor.writeOne(this, transformation);
    }

    public void writeOne(String accessor, Accessor.Function transformation) {
        writeOne(Accessor.parse(accessor), transformation);
    }

    public void writeOne(Accessor accessor, Accessor.Function transformation, boolean failureAcceptable) {
        accessor.writeOne(this, transformation, failureAcceptable);
    }
    
    public void writeOne(String accessor, Accessor.Function transformation, boolean failureAcceptable) {
        writeOne(Accessor.parse(accessor), transformation, failureAcceptable);
    }
}
