/*
 * Copyright 2013 Andy Boothe
 * 
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 * 
 *        http://www.apache.org/licenses/LICENSE-2.0
 * 
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.sigpwned.jsonification.processing;

import java.io.IOException;

import com.sigpwned.jsonification.JsonProcessor;
import com.sigpwned.jsonification.util.AbstractJsonProcessor;

public class LoggingJsonProcessor<U,T extends JsonProcessor<U>> extends AbstractJsonProcessor<U> {
	private T inner;
	
	public LoggingJsonProcessor(T inner) {
		this.inner = inner;
	}
	
	public T getInner() {
		return inner;
	}
	
	protected void log(String message, Object... args) {
		// TODO Generalize to support more types of logging
		System.err.printf(message, args);
		System.err.println();
	}

	public void beginJson() throws IOException {
		log("beginJson()");
		getInner().beginJson();
	}

	public U endJson() throws IOException {
		log("endJson()");
		return getInner().endJson();
	}

	public void beginObject(Coordinate coordinate) throws IOException {
		log("beginObject("+coordinate+")");
		getInner().beginObject(coordinate);
	}

	public void endObject() throws IOException {
		log("endObject");
		getInner().endObject();
	}

	public void beginList(Coordinate coordinate) throws IOException {
		log("beginList("+coordinate+")");
		getInner().beginList(coordinate);
	}

	public void endList() throws IOException {
		log("endList");
		getInner().endList();
	}

	public void value(Coordinate coordinate, boolean value) throws IOException {
		log("value("+coordinate+", "+value+")");
		getInner().value(coordinate, value);
	}

	public void value(Coordinate coordinate, long value) throws IOException {
		log("value("+coordinate+", "+value+")");
		getInner().value(coordinate, value);
	}

	public void value(Coordinate coordinate, double value) throws IOException {
		log("value("+coordinate+", "+value+")");
		getInner().value(coordinate, value);
	}

	public void value(Coordinate coordinate, String value) throws IOException {
		log("value("+coordinate+", "+value+")");
		getInner().value(coordinate, value);
	}

	public void nullValue(Coordinate coordinate) throws IOException {
		log("nullValue("+coordinate+")");
		getInner().nullValue(coordinate);
	}
}
