/*
 * Copyright 2013 Andy Boothe
 * 
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 * 
 *        http://www.apache.org/licenses/LICENSE-2.0
 * 
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.sigpwned.jsonification.serialization.serialize.approach;

import java.io.IOException;

import com.sigpwned.jsonification.Json;
import com.sigpwned.jsonification.JsonProcessor;
import com.sigpwned.jsonification.JsonSerializer;
import com.sigpwned.jsonification.manipulation.JsonFragmentBuilder;
import com.sigpwned.jsonification.model.JsonBoolean;
import com.sigpwned.jsonification.model.JsonFloat;
import com.sigpwned.jsonification.model.JsonInt;
import com.sigpwned.jsonification.model.JsonList;
import com.sigpwned.jsonification.model.JsonNumber;
import com.sigpwned.jsonification.model.JsonObject;
import com.sigpwned.jsonification.model.JsonString;
import com.sigpwned.jsonification.model.JsonValue;
import com.sigpwned.jsonification.serialization.serialize.JsonSerializeApproach;
import com.sigpwned.jsonification.x.InternalErrorJsonException;

public class JsonValueJsonSerializeApproach extends JsonSerializeApproach {

    @Override
    public boolean serialize(JsonSerializer serializer, JsonFragmentBuilder builder, Object value) throws IOException {
        boolean result;
        
        if(value instanceof JsonValue) {
            JsonValue v=(JsonValue) value;
            value(builder, null, v);
            result = true;
        }
        else
            result = false;
        
        return result;
    }
    
    protected void value(JsonFragmentBuilder builder, JsonProcessor.Coordinate coordinate, JsonValue v) throws IOException {
        if(v == Json.NULL) {
            if(coordinate != null)
                builder.nullValue(coordinate);
            else
                builder.nullValue();
        } else
        if(v instanceof JsonInt) {
            long value=v.asInt().longVal();
            if(coordinate != null)
                builder.value(coordinate, value);
            else
                builder.value(value);
        } else
        if(v instanceof JsonFloat) {
            double value=v.asFloat().doubleVal();
            if(coordinate != null)
                builder.value(coordinate, value);
            else
                builder.value(value);
        } else
        if(v instanceof JsonNumber) {
            JsonNumber n=v.asNumber();
            if(n.getOriginalType() == JsonNumber.OriginalType.INT) {
                long value=v.asInt().longVal();
                if(coordinate != null)
                    builder.value(coordinate, value);
                else
                    builder.value(value);
            } else
            if(n.getOriginalType() == JsonNumber.OriginalType.FLOAT) {
                double value=v.asFloat().doubleVal();
                if(coordinate != null)
                    builder.value(coordinate, value);
                else
                    builder.value(value);
            }
            else
                throw new InternalErrorJsonException("Unhandled OriginalType: "+n.getOriginalType());
        } else
        if(v instanceof JsonString) {
            String value=v.asString().stringVal();
            if(coordinate != null)
                builder.value(coordinate, value);
            else
                builder.value(value);
        } else
        if(v instanceof JsonBoolean) {
            boolean value=v.asBoolean().booleanVal();
            if(coordinate != null)
                builder.value(coordinate, value);
            else
                builder.value(value);
        } else
        if(v instanceof JsonObject) {
            JsonObject o=v.asObject();
            
            if(coordinate != null)
                builder.beginObject(coordinate);
            else
                builder.beginObject();
            try {
                for(String key : o.keySet())
                    value(builder, new JsonProcessor.Name(key), o.get(key));
            }
            finally {
                builder.endObject();
            }
        } else
        if(v instanceof JsonList) {
            JsonList l=v.asList();
            
            if(coordinate != null)
                builder.beginList(coordinate);
            else
                builder.beginList();
            try {
                for(JsonValue value : l)
                    value(builder, null, value);
            }
            finally {
                builder.endObject();
            }
        }
    }
}
