/*
 * Copyright 2013 Andy Boothe
 * 
 *    Licensed under the Apache License, Version 2.0 (the "License");
 *    you may not use this file except in compliance with the License.
 *    You may obtain a copy of the License at
 * 
 *        http://www.apache.org/licenses/LICENSE-2.0
 * 
 *    Unless required by applicable law or agreed to in writing, software
 *    distributed under the License is distributed on an "AS IS" BASIS,
 *    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *    See the License for the specific language governing permissions and
 *    limitations under the License.
 */
package com.sigpwned.jsonification.serialization.serialize.approach;


import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.Arrays;

import com.sigpwned.jsonification.JsonSerializer;
import com.sigpwned.jsonification.manipulation.JsonFragmentBuilder;
import com.sigpwned.jsonification.serialization.JsonSerializationException;
import com.sigpwned.jsonification.serialization.serialize.JsonSerializeApproach;

public class ToJsonMethodJsonSerializeApproach extends JsonSerializeApproach {
	@Override
	public boolean serialize(JsonSerializer serializer, JsonFragmentBuilder builder, Object value) {
		boolean result;
		
		Method toJson=findMethod(value.getClass(), boolean.class, Arrays.asList("toJson"), JsonFragmentBuilder.class);
		if(toJson != null) {
			try {
				result = (Boolean) toJson.invoke(value, builder);
			}
			catch (IllegalArgumentException e) {
				JsonSerializationException e2=serializer.getExceptionPolicy().format(e);
				if(e2 != null)
					throw e2;
				result = false;
			}
			catch (IllegalAccessException e) {
				JsonSerializationException e2=serializer.getExceptionPolicy().format(e);
				if(e2 != null)
					throw e2;
				result = false;
			}
			catch (InvocationTargetException e) {
				JsonSerializationException e2=serializer.getExceptionPolicy().format(e);
				if(e2 != null)
					throw e2;
				result = false;
			}
		}
		else
			result = false;
		
		return result;
	}
}
